/***************************************************************************
 * Gens: Reverb Effect. (S2HD)                                             *
 *                                                                         *
 * Copyright (c) 2009 by David Korth                                       *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

#include "audio_reverb.h"
#include "audio.h"

#include <stdlib.h>
#include <string.h>


static short* echo_buffer = NULL;
static int echo_buffer_size = 0;


void audio_reverb(short *buf, int len)
{
	int sz = len * (audio_get_stereo() ? 2 : 1);
	if (echo_buffer_size != sz)
	{
		echo_buffer_size = sz;
		if (echo_buffer)
			free(echo_buffer);
		
		echo_buffer = (short*)malloc(sz * sizeof(*echo_buffer));
	}
	
	// Copy the echo buffer.
	memcpy(echo_buffer, buf, echo_buffer_size * sizeof(*echo_buffer));
	
	// Apply the echo twice:
	// - once at 1/3 seconds, at half volume.
	// - once at 2/3 seconds, at quarter volume.
	int src, dest;
	for (src = 0, dest = (sz / 3); dest < sz; src++, dest++)
	{
		int sample = buf[dest];
		sample += (echo_buffer[src] / 2);
		if (sample > 0x7FFF)
			sample = 0x7FFF;
		if (sample < -0x7FFF)
			sample = -0x7FFF;
		buf[dest] = sample;
	}
	
	for (src = 0, dest = (sz * 2 / 3); dest < sz; src++, dest++)
	{
		int sample = buf[dest];
		sample += (echo_buffer[src] / 4);
		if (sample > 0x7FFF)
			sample = 0x7FFF;
		if (sample < -0x7FFF)
			sample = -0x7FFF;
		buf[dest] = sample;
	}
}
