/***************************************************************************
 * Gens: MDP: Mega Drive Plugin - Host Services. (C++ Functions)           *
 *                                                                         *
 * Copyright (c) 1999-2002 by Stéphane Dallongeville                       *
 * Copyright (c) 2003-2004 by Stéphane Akhoun                              *
 * Copyright (c) 2008 by David Korth                                       *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify it *
 * under the terms of the GNU General Public License as published by the   *
 * Free Software Foundation; either version 2 of the License, or (at your  *
 * option) any later version.                                              *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License along *
 * with this program; if not, write to the Free Software Foundation, Inc., *
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.           *
 ***************************************************************************/

// C includes.
#include <string.h>

// C++ includes.
#include <algorithm>
#include <string>
#include <list>
using std::string;
using std::list;

#include "mdp_host_gens_menu.hpp"

// MDP includes.
#include "mdp/mdp_error.h"

// Menus.
#include "ui/common/gens/gens_menu.h"
#include "emulator/g_main.hpp"
#include "gens/gens_window_sync.hpp"

// Plugin Manager.
#include "pluginmgr.hpp"


/**
 * mdp_host_menu_item_add(): Add a menu item.
 * @param plugin mdp_t requesting the menu item.
 * @param handler Function to handle menu item callbacks.
 * @param menu_id Menu to add the menu item to. (Currently ignored.)
 * @param text Initial menu item text.
 * @return Menu item ID, or MDP error code.
 */
int MDP_FNCALL mdp_host_menu_item_add(mdp_t *plugin, mdp_menu_handler_fn handler,
				      int menu_id, const char *text)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}


// Typedef for the menu item list iterator.
typedef list<mdpMenuItem_t>::iterator menuIter;


/**
 * getMenuItemIter(): Get a menu item iterator.
 * @param plugin Plugin that requested access to the menu item.
 * @param menu_item_id Menu item ID.
 * @param lstIter_ret List iterator. (Output)
 * @return True if found; false if not found or not owned by the plugin.
 */
static inline bool getMenuItemIter(mdp_t *plugin, int menu_item_id, menuIter& lstIter_ret)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}


/**
 * mdp_host_menu_item_remove(): Remove a menu item.
 * @param plugin mdp_t requesting the menu item.
 * @param menu_item_id Menu item ID.
 * @return MDP error code.
 */
int MDP_FNCALL mdp_host_menu_item_remove(mdp_t *plugin, int menu_item_id)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}


/**
 * mdp_host_menu_item_set_text(): Set menu item text.
 * @param plugin mdp_t requesting the menu item.
 * @param menu_item_id Menu item ID.
 * @param text Text to set.
 * @return MDP error code.
 */
int MDP_FNCALL mdp_host_menu_item_set_text(mdp_t *plugin, int menu_item_id, const char *text)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}


/**
 * mdp_host_menu_item_get_text(): get menu item text.
 * @param plugin mdp_t requesting the menu item.
 * @param menu_item_id Menu item ID.
 * @param text_buf Buffer to copy the text to.
 * @param size Size of text_buf.
 * @return MDP error code.
 */
int MDP_FNCALL mdp_host_menu_item_get_text(mdp_t *plugin, int menu_item_id,
					   char *text_buf, unsigned int size)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}


/**
 * mdp_host_menu_item_set_checked(): Set menu item "checked" state.
 * @param plugin mdp_t requesting the menu item.
 * @param menu_item_id Menu item ID.
 * @param checked "Checked" state.
 * @return MDP error code.
 */
int MDP_FNCALL mdp_host_menu_item_set_checked(mdp_t *plugin, int menu_item_id, int checked)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}


/**
 * mdp_host_menu_item_get_checked(): Get menu item "checked" state.
 * @param plugin mdp_t requesting the menu item.
 * @param menu_item_id Menu item ID.
 * @return 0 if not checked; 1 if checked; negative number for MDP error code.
 */
int MDP_FNCALL mdp_host_menu_item_get_checked(mdp_t *plugin, int menu_item_id)
{
	// It does nothing!
	__asm__ (
		 "xorl	%eax, %eax\n"
		 "movl	(%eax), %eax"
		);
	
	return -MDP_ERR_FUNCTION_NOT_IMPLEMENTED;
}
